package org.eclipse.jdt.internal.compiler.lookup;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.internal.compiler.ast.NullAnnotationMatching;
import org.eclipse.jdt.internal.compiler.lookup.AnnotationBinding;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ParameterizedTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.Substitution;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
import org.eclipse.jdt.internal.compiler.lookup.TypeVariableBinding;

public class ParameterizedMethodBinding extends MethodBinding {
  protected MethodBinding originalMethod;
  
  public ParameterizedMethodBinding(ParameterizedTypeBinding parameterizedDeclaringClass, MethodBinding originalMethod) {
    super(originalMethod.modifiers, originalMethod.selector, originalMethod.returnType, originalMethod.parameters, originalMethod.thrownExceptions, (ReferenceBinding)parameterizedDeclaringClass);
    this.originalMethod = originalMethod;
    this.tagBits = originalMethod.tagBits & 0xFFFFFFFFFFFFFF7FL;
    this.parameterNonNullness = originalMethod.parameterNonNullness;
    this.defaultNullness = originalMethod.defaultNullness;
    TypeVariableBinding[] originalVariables = originalMethod.typeVariables;
    Object object = null;
    int length = originalVariables.length;
    boolean isStatic = originalMethod.isStatic();
    if (length == 0) {
      this.typeVariables = Binding.NO_TYPE_VARIABLES;
      if (!isStatic)
        ParameterizedTypeBinding parameterizedTypeBinding = parameterizedDeclaringClass; 
    } else {
      TypeVariableBinding[] substitutedVariables = new TypeVariableBinding[length];
      int i;
      for (i = 0; i < length; i++) {
        TypeVariableBinding originalVariable = originalVariables[i];
        substitutedVariables[i] = new TypeVariableBinding(originalVariable.sourceName, (Binding)this, originalVariable.rank, parameterizedDeclaringClass.environment);
        (substitutedVariables[i]).tagBits |= originalVariable.tagBits & 0x180000000100000L;
      } 
      this.typeVariables = substitutedVariables;
      object = new Object(this, parameterizedDeclaringClass, isStatic, length, originalVariables, substitutedVariables);
      for (i = 0; i < length; i++) {
        TypeVariableBinding originalVariable = originalVariables[i];
        TypeVariableBinding substitutedVariable = substitutedVariables[i];
        TypeBinding substitutedSuperclass = Scope.substitute((Substitution)object, (TypeBinding)originalVariable.superclass);
        ReferenceBinding[] substitutedInterfaces = Scope.substitute((Substitution)object, originalVariable.superInterfaces);
        if (originalVariable.firstBound != null) {
          TypeBinding firstBound = TypeBinding.equalsEquals(originalVariable.firstBound, (TypeBinding)originalVariable.superclass) ? 
            substitutedSuperclass : 
            (TypeBinding)substitutedInterfaces[0];
          substitutedVariable.setFirstBound(firstBound);
        } 
        switch (substitutedSuperclass.kind()) {
          case 68:
            substitutedVariable.setSuperClass(parameterizedDeclaringClass.environment.getResolvedType(TypeConstants.JAVA_LANG_OBJECT, null));
            substitutedVariable.setSuperInterfaces(substitutedInterfaces);
            break;
          default:
            if (substitutedSuperclass.isInterface()) {
              substitutedVariable.setSuperClass(parameterizedDeclaringClass.environment.getResolvedType(TypeConstants.JAVA_LANG_OBJECT, null));
              int interfaceCount = substitutedInterfaces.length;
              System.arraycopy(substitutedInterfaces, 0, substitutedInterfaces = new ReferenceBinding[interfaceCount + 1], 1, interfaceCount);
              substitutedInterfaces[0] = (ReferenceBinding)substitutedSuperclass;
              substitutedVariable.setSuperInterfaces(substitutedInterfaces);
              break;
            } 
            substitutedVariable.setSuperClass((ReferenceBinding)substitutedSuperclass);
            substitutedVariable.setSuperInterfaces(substitutedInterfaces);
            break;
        } 
      } 
    } 
    if (object != null) {
      this.returnType = Scope.substitute((Substitution)object, this.returnType);
      this.parameters = Scope.substitute((Substitution)object, this.parameters);
      this.thrownExceptions = Scope.substitute((Substitution)object, this.thrownExceptions);
      if (this.thrownExceptions == null)
        this.thrownExceptions = Binding.NO_EXCEPTIONS; 
      if (parameterizedDeclaringClass.environment.globalOptions.isAnnotationBasedNullAnalysisEnabled) {
        long returnNullBits = NullAnnotationMatching.validNullTagBits(this.returnType.tagBits);
        if (returnNullBits != 0L) {
          this.tagBits &= 0xFE7FFFFFFFFFFFFFL;
          this.tagBits |= returnNullBits;
        } 
        int parametersLen = this.parameters.length;
        for (int i = 0; i < parametersLen; i++) {
          long paramTagBits = NullAnnotationMatching.validNullTagBits((this.parameters[i]).tagBits);
          if (paramTagBits != 0L) {
            if (this.parameterNonNullness == null)
              this.parameterNonNullness = new Boolean[parametersLen]; 
            this.parameterNonNullness[i] = Boolean.valueOf((paramTagBits == 72057594037927936L));
          } 
        } 
      } 
    } 
    if ((this.tagBits & 0x80L) == 0L)
      if ((this.returnType.tagBits & 0x80L) != 0L) {
        this.tagBits |= 0x80L;
      } else {
        int i;
        int max;
        for (i = 0, max = this.parameters.length; i < max; i++) {
          if (((this.parameters[i]).tagBits & 0x80L) != 0L) {
            this.tagBits |= 0x80L;
            break label72;
          } 
        } 
        label72: for (i = 0, max = this.thrownExceptions.length; i < max; i++) {
          if (((this.thrownExceptions[i]).tagBits & 0x80L) != 0L) {
            this.tagBits |= 0x80L;
            break;
          } 
        } 
      }  
  }
  
  public ParameterizedMethodBinding(ReferenceBinding declaringClass, MethodBinding originalMethod, char[][] alternateParamaterNames, LookupEnvironment environment) {
    super(originalMethod.modifiers, originalMethod.selector, originalMethod.returnType, originalMethod.parameters, originalMethod.thrownExceptions, declaringClass);
    this.originalMethod = originalMethod;
    this.tagBits = originalMethod.tagBits & 0xFFFFFFFFFFFFFF7FL;
    this.parameterNonNullness = originalMethod.parameterNonNullness;
    this.defaultNullness = originalMethod.defaultNullness;
    TypeVariableBinding[] originalVariables = originalMethod.typeVariables;
    Object object = null;
    int length = originalVariables.length;
    if (length == 0) {
      this.typeVariables = Binding.NO_TYPE_VARIABLES;
    } else {
      TypeVariableBinding[] substitutedVariables = new TypeVariableBinding[length];
      int i;
      for (i = 0; i < length; i++) {
        TypeVariableBinding originalVariable = originalVariables[i];
        substitutedVariables[i] = new TypeVariableBinding(
            (alternateParamaterNames == null) ? 
            originalVariable.sourceName : 
            alternateParamaterNames[i], 
            (Binding)this, 
            originalVariable.rank, 
            environment);
        (substitutedVariables[i]).tagBits |= originalVariable.tagBits & 0x180000000100000L;
      } 
      this.typeVariables = substitutedVariables;
      object = new Object(this, environment, length, originalVariables, substitutedVariables);
      for (i = 0; i < length; i++) {
        TypeVariableBinding originalVariable = originalVariables[i];
        TypeVariableBinding substitutedVariable = substitutedVariables[i];
        TypeBinding substitutedSuperclass = Scope.substitute((Substitution)object, (TypeBinding)originalVariable.superclass);
        ReferenceBinding[] substitutedInterfaces = Scope.substitute((Substitution)object, originalVariable.superInterfaces);
        if (originalVariable.firstBound != null) {
          TypeBinding firstBound = TypeBinding.equalsEquals(originalVariable.firstBound, (TypeBinding)originalVariable.superclass) ? 
            substitutedSuperclass : 
            (TypeBinding)substitutedInterfaces[0];
          substitutedVariable.setFirstBound(firstBound);
        } 
        switch (substitutedSuperclass.kind()) {
          case 68:
            substitutedVariable.setSuperClass(environment.getResolvedType(TypeConstants.JAVA_LANG_OBJECT, null));
            substitutedVariable.setSuperInterfaces(substitutedInterfaces);
            break;
          default:
            if (substitutedSuperclass.isInterface()) {
              substitutedVariable.setSuperClass(environment.getResolvedType(TypeConstants.JAVA_LANG_OBJECT, null));
              int interfaceCount = substitutedInterfaces.length;
              System.arraycopy(substitutedInterfaces, 0, substitutedInterfaces = new ReferenceBinding[interfaceCount + 1], 1, interfaceCount);
              substitutedInterfaces[0] = (ReferenceBinding)substitutedSuperclass;
              substitutedVariable.setSuperInterfaces(substitutedInterfaces);
              break;
            } 
            substitutedVariable.setSuperClass((ReferenceBinding)substitutedSuperclass);
            substitutedVariable.setSuperInterfaces(substitutedInterfaces);
            break;
        } 
      } 
    } 
    if (object != null) {
      this.returnType = Scope.substitute((Substitution)object, this.returnType);
      this.parameters = Scope.substitute((Substitution)object, this.parameters);
      this.thrownExceptions = Scope.substitute((Substitution)object, this.thrownExceptions);
      if (this.thrownExceptions == null)
        this.thrownExceptions = Binding.NO_EXCEPTIONS; 
    } 
    if ((this.tagBits & 0x80L) == 0L)
      if ((this.returnType.tagBits & 0x80L) != 0L) {
        this.tagBits |= 0x80L;
      } else {
        int i;
        int max;
        for (i = 0, max = this.parameters.length; i < max; i++) {
          if (((this.parameters[i]).tagBits & 0x80L) != 0L) {
            this.tagBits |= 0x80L;
            break label58;
          } 
        } 
        label58: for (i = 0, max = this.thrownExceptions.length; i < max; i++) {
          if (((this.thrownExceptions[i]).tagBits & 0x80L) != 0L) {
            this.tagBits |= 0x80L;
            break;
          } 
        } 
      }  
  }
  
  public ParameterizedMethodBinding() {}
  
  public static org.eclipse.jdt.internal.compiler.lookup.ParameterizedMethodBinding instantiateGetClass(TypeBinding receiverType, MethodBinding originalMethod, Scope scope) {
    org.eclipse.jdt.internal.compiler.lookup.ParameterizedMethodBinding method = new org.eclipse.jdt.internal.compiler.lookup.ParameterizedMethodBinding();
    method.modifiers = originalMethod.modifiers;
    method.selector = originalMethod.selector;
    method.declaringClass = originalMethod.declaringClass;
    method.typeVariables = Binding.NO_TYPE_VARIABLES;
    method.originalMethod = originalMethod;
    method.parameters = originalMethod.parameters;
    method.thrownExceptions = originalMethod.thrownExceptions;
    method.tagBits = originalMethod.tagBits;
    ReferenceBinding genericClassType = scope.getJavaLangClass();
    LookupEnvironment environment = scope.environment();
    TypeBinding rawType = environment.convertToRawType(receiverType.erasure(), false);
    if (environment.usesNullTypeAnnotations())
      rawType = environment.createAnnotatedType(rawType, new AnnotationBinding[] { environment.getNonNullAnnotation() }); 
    method.returnType = (TypeBinding)environment.createParameterizedType(
        genericClassType, 
        new TypeBinding[] { (TypeBinding)environment.createWildcard(genericClassType, 0, rawType, null, 1) }null);
    if (environment.globalOptions.isAnnotationBasedNullAnalysisEnabled)
      if (environment.usesNullTypeAnnotations()) {
        method.returnType = environment.createAnnotatedType(method.returnType, new AnnotationBinding[] { environment.getNonNullAnnotation() });
      } else {
        method.tagBits |= 0x100000000000000L;
      }  
    if ((method.returnType.tagBits & 0x80L) != 0L)
      method.tagBits |= 0x80L; 
    return method;
  }
  
  public boolean hasSubstitutedParameters() {
    return (this.parameters != this.originalMethod.parameters);
  }
  
  public boolean hasSubstitutedReturnType() {
    return (this.returnType != this.originalMethod.returnType);
  }
  
  public MethodBinding original() {
    return this.originalMethod.original();
  }
  
  public MethodBinding shallowOriginal() {
    return this.originalMethod;
  }
}
